% num = match(image1, image2)
%
% This function reads two images, finds their SIFT features, and
%   displays lines connecting the matched keypoints.  A match is accepted
%   only if its distance is less than distRatio times the distance to the
%   second closest match.
% It returns the number of matches displayed.
%
% Example: match('scene.pgm','book.pgm');

function [input_points, base_points] = match(image1, image2)

[des1 nb loc1]=loadFeatures('img2.desc');
[des2 nb loc2]=loadFeatures('img3.desc');
% % Find SIFT keypoints for each image
% [im1, des1, loc1] = sift(image1);
% [im2, des2, loc2] = sift(image2);

% For efficiency in Matlab, it is cheaper to compute dot products between
%  unit vectors rather than Euclidean distances.  Note that the ratio of 
%  angles (acos of dot products of unit vectors) is a close approximation
%  to the ratio of Euclidean distances for small angles.
%
% distRatio: Only keep matches in which the ratio of vector angles from the
%   nearest to second nearest neighbor is less than distRatio.
distRatio = 0.7;   

% For each descriptor in the first image, select its match to second image.
des2t = des2';    
normdes2t = normr(des2t);  % Precompute matrix transpose
for i = 1 : size(des1,2)
   dotprods = normdes2t * normc(des1(:,i));        % Computes vector of dot products
   [vals,indx] = sort(acosd(dotprods));  % Take inverse cosine and sort results
   
   % Check if nearest neighbor has angle less than distRatio times 2nd.
   if vals(1)<20 & (vals(1) < distRatio * vals(2))
      match(i) = indx(1);
   else
      match(i) = 0;
   end
end

I1=imread(image1); I2=imread(image2);
% Create a new image showing the two images side by side.
im3 = appendimages(I1,I2);

% Show a figure with lines joining the accepted matches.
figure('Position', [100 100 size(im3,2) size(im3,1)]);
colormap('gray');
imagesc(im3);
hold on;
cols1 = size(I1,2);
input_points=[]; base_points=[];
for i = 1: size(des1,1)
  if (match(i) > 0)
    input_points=[input_points; loc2(match(i),2) loc2(match(i),1)];
    base_points=[base_points; loc1(i,2) loc1(i,1)];
    line([loc1(i,1) loc2(match(i),1)+cols1], ...
         [loc1(i,2) loc2(match(i),2)], 'Color', 'c');
  end
end
hold off;
num = sum(match > 0);
fprintf('Found %d matches.\n', num);




